@@ -13,8 +13,8 @@ namespace cv {
 
 Mat _InputArray::getMat_(int i) const
 {
-    int k = kind();
-    int accessFlags = flags & ACCESS_MASK;
+    _InputArray::KindFlag k = kind();
+    AccessFlag accessFlags = flags & ACCESS_MASK;
 
     if( k == MAT )
     {
@@ -125,8 +125,8 @@ Mat _InputArray::getMat_(int i) const
 
 UMat _InputArray::getUMat(int i) const
 {
-    int k = kind();
-    int accessFlags = flags & ACCESS_MASK;
+    _InputArray::KindFlag k = kind();
+    AccessFlag accessFlags = flags & ACCESS_MASK;
 
     if( k == UMAT )
     {
@@ -157,8 +157,8 @@ UMat _InputArray::getUMat(int i) const
 
 void _InputArray::getMatVector(std::vector<Mat>& mv) const
 {
-    int k = kind();
-    int accessFlags = flags & ACCESS_MASK;
+    _InputArray::KindFlag k = kind();
+    AccessFlag accessFlags = flags & ACCESS_MASK;
 
     if( k == MAT )
     {
@@ -254,8 +254,8 @@ void _InputArray::getMatVector(std::vector<Mat>& mv) const
 
 void _InputArray::getUMatVector(std::vector<UMat>& umv) const
 {
-    int k = kind();
-    int accessFlags = flags & ACCESS_MASK;
+    _InputArray::KindFlag k = kind();
+    AccessFlag accessFlags = flags & ACCESS_MASK;
 
     if( k == NONE )
     {
@@ -317,7 +317,7 @@ void _InputArray::getUMatVector(std::vector<UMat>& umv) const
 cuda::GpuMat _InputArray::getGpuMat() const
 {
 #ifdef HAVE_CUDA
-    int k = kind();
+    _InputArray::KindFlag k = kind();
 
     if (k == CUDA_GPU_MAT)
     {
@@ -347,7 +347,7 @@ cuda::GpuMat _InputArray::getGpuMat() const
 void _InputArray::getGpuMatVector(std::vector<cuda::GpuMat>& gpumv) const
 {
 #ifdef HAVE_CUDA
-    int k = kind();
+    _InputArray::KindFlag k = kind();
     if (k == STD_VECTOR_CUDA_GPU_MAT)
     {
         gpumv = *(std::vector<cuda::GpuMat>*)obj;
@@ -359,7 +359,7 @@ void _InputArray::getGpuMatVector(std::vector<cuda::GpuMat>& gpumv) const
 }
 ogl::Buffer _InputArray::getOGlBuffer() const
 {
-    int k = kind();
+    _InputArray::KindFlag k = kind();
 
     CV_Assert(k == OPENGL_BUFFER);
 
@@ -367,15 +367,11 @@ ogl::Buffer _InputArray::getOGlBuffer() const
     return *gl_buf;
 }
 
-int _InputArray::kind() const
+_InputArray::KindFlag _InputArray::kind() const
 {
-    int k = flags & KIND_MASK;
+    KindFlag k = flags & KIND_MASK;
 #if CV_VERSION_MAJOR < 5
     CV_DbgAssert(k != EXPR);
-#if CV_VERSION_MAJOR < 4
-    if (k == STD_ARRAY)
-        k = MATX;
-#endif
     CV_DbgAssert(k != STD_ARRAY);
 #endif
     return k;
@@ -393,7 +389,7 @@ int _InputArray::cols(int i) const
 
 Size _InputArray::size(int i) const
 {
-    int k = kind();
+    _InputArray::KindFlag k = kind();
 
     if( k == MAT )
     {
@@ -513,7 +509,8 @@ Size _InputArray::size(int i) const
 
 int _InputArray::sizend(int* arrsz, int i) const
 {
-    int j, d=0, k = kind();
+    int j, d = 0;
+    _InputArray::KindFlag k = kind();
 
     if( k == NONE )
         ;
@@ -582,7 +579,7 @@ int _InputArray::sizend(int* arrsz, int i) const
 
 bool _InputArray::sameSize(const _InputArray& arr) const
 {
-    int k1 = kind(), k2 = arr.kind();
+    _InputArray::KindFlag k1 = kind(), k2 = arr.kind();
     Size sz1;
 
     if( k1 == MAT )
@@ -616,7 +613,7 @@ bool _InputArray::sameSize(const _InputArray& arr) const
 
 int _InputArray::dims(int i) const
 {
-    int k = kind();
+    _InputArray::KindFlag k = kind();
 
     if( k == MAT )
     {
@@ -707,7 +704,7 @@ int _InputArray::dims(int i) const
 
 size_t _InputArray::total(int i) const
 {
-    int k = kind();
+    _InputArray::KindFlag k = kind();
 
     if( k == MAT )
     {
@@ -756,7 +753,7 @@ size_t _InputArray::total(int i) const
 
 int _InputArray::type(int i) const
 {
-    int k = kind();
+    _InputArray::KindFlag k = kind();
 
     if( k == MAT )
         return ((const Mat*)obj)->type();
@@ -846,7 +843,7 @@ int _InputArray::channels(int i) const
 
 bool _InputArray::empty() const
 {
-    int k = kind();
+    _InputArray::KindFlag k = kind();
 
     if( k == MAT )
         return ((const Mat*)obj)->empty();
@@ -915,7 +912,7 @@ bool _InputArray::empty() const
 
 bool _InputArray::isContinuous(int i) const
 {
-    int k = kind();
+    _InputArray::KindFlag k = kind();
 
     if( k == MAT )
         return i < 0 ? ((const Mat*)obj)->isContinuous() : true;
@@ -951,12 +948,12 @@ bool _InputArray::isContinuous(int i) const
     if( k == CUDA_GPU_MAT )
       return i < 0 ? ((const cuda::GpuMat*)obj)->isContinuous() : true;
 
-    CV_Error(CV_StsNotImplemented, "Unknown/unsupported array type");
+    CV_Error(cv::Error::StsNotImplemented, "Unknown/unsupported array type");
 }
 
 bool _InputArray::isSubmatrix(int i) const
 {
-    int k = kind();
+    _InputArray::KindFlag k = kind();
 
     if( k == MAT )
         return i < 0 ? ((const Mat*)obj)->isSubmatrix() : false;
@@ -989,12 +986,12 @@ bool _InputArray::isSubmatrix(int i) const
         return vv[i].isSubmatrix();
     }
 
-    CV_Error(CV_StsNotImplemented, "");
+    CV_Error(cv::Error::StsNotImplemented, "");
 }
 
 size_t _InputArray::offset(int i) const
 {
-    int k = kind();
+    _InputArray::KindFlag k = kind();
 
     if( k == MAT )
     {
@@ -1054,7 +1051,7 @@ size_t _InputArray::offset(int i) const
 
 size_t _InputArray::step(int i) const
 {
-    int k = kind();
+    _InputArray::KindFlag k = kind();
 
     if( k == MAT )
     {
@@ -1110,7 +1107,7 @@ size_t _InputArray::step(int i) const
 
 void _InputArray::copyTo(const _OutputArray& arr) const
 {
-    int k = kind();
+    _InputArray::KindFlag k = kind();
 
     if( k == NONE )
         arr.release();
@@ -1131,7 +1128,7 @@ void _InputArray::copyTo(const _OutputArray& arr) const
 
 void _InputArray::copyTo(const _OutputArray& arr, const _InputArray & mask) const
 {
-    int k = kind();
+    _InputArray::KindFlag k = kind();
 
     if( k == NONE )
         arr.release();
@@ -1160,9 +1157,9 @@ bool _OutputArray::fixedType() const
     return (flags & FIXED_TYPE) == FIXED_TYPE;
 }
 
-void _OutputArray::create(Size _sz, int mtype, int i, bool allowTransposed, int fixedDepthMask) const
+void _OutputArray::create(Size _sz, int mtype, int i, bool allowTransposed, _OutputArray::DepthMask fixedDepthMask) const
 {
-    int k = kind();
+    _InputArray::KindFlag k = kind();
     if( k == MAT && i < 0 && !allowTransposed && fixedDepthMask == 0 )
     {
         CV_Assert(!fixedSize() || ((Mat*)obj)->size.operator()() == _sz);
@@ -1214,9 +1211,9 @@ void _OutputArray::create(Size _sz, int mtype, int i, bool allowTransposed, int
     create(2, sizes, mtype, i, allowTransposed, fixedDepthMask);
 }
 
-void _OutputArray::create(int _rows, int _cols, int mtype, int i, bool allowTransposed, int fixedDepthMask) const
+void _OutputArray::create(int _rows, int _cols, int mtype, int i, bool allowTransposed, _OutputArray::DepthMask fixedDepthMask) const
 {
-    int k = kind();
+    _InputArray::KindFlag k = kind();
     if( k == MAT && i < 0 && !allowTransposed && fixedDepthMask == 0 )
     {
         CV_Assert(!fixedSize() || ((Mat*)obj)->size.operator()() == Size(_cols, _rows));
@@ -1269,7 +1266,7 @@ void _OutputArray::create(int _rows, int _cols, int mtype, int i, bool allowTran
 }
 
 void _OutputArray::create(int d, const int* sizes, int mtype, int i,
-                          bool allowTransposed, int fixedDepthMask) const
+                          bool allowTransposed, _OutputArray::DepthMask fixedDepthMask) const
 {
     int sizebuf[2];
     if(d == 1)
@@ -1279,7 +1276,7 @@ void _OutputArray::create(int d, const int* sizes, int mtype, int i,
         sizebuf[1] = 1;
         sizes = sizebuf;
     }
-    int k = kind();
+    _InputArray::KindFlag k = kind();
     mtype = CV_MAT_TYPE(mtype);
 
     if( k == MAT )
@@ -1469,14 +1466,14 @@ void _OutputArray::create(int d, const int* sizes, int mtype, int i,
             ((std::vector<Vec<int, 128> >*)v)->resize(len);
             break;
         default:
-            CV_Error_(CV_StsBadArg, ("Vectors with element size %d are not supported. Please, modify OutputArray::create()\n", esz));
+            CV_Error_(cv::Error::StsBadArg, ("Vectors with element size %d are not supported. Please, modify OutputArray::create()\n", esz));
         }
         return;
     }
 
     if( k == NONE )
     {
-        CV_Error(CV_StsNullPtr, "create() called for the missing output array" );
+        CV_Error(cv::Error::StsNullPtr, "create() called for the missing output array" );
     }
 
     if( k == STD_VECTOR_MAT )
@@ -1669,7 +1666,7 @@ void _OutputArray::release() const
 {
     CV_Assert(!fixedSize());
 
-    int k = kind();
+    _InputArray::KindFlag k = kind();
 
     if( k == MAT )
     {
@@ -1753,7 +1750,7 @@ void _OutputArray::release() const
 
 void _OutputArray::clear() const
 {
-    int k = kind();
+    _InputArray::KindFlag k = kind();
 
     if( k == MAT )
     {
@@ -1772,7 +1769,7 @@ bool _OutputArray::needed() const
 
 Mat& _OutputArray::getMatRef(int i) const
 {
-    int k = kind();
+    _InputArray::KindFlag k = kind();
     if( i < 0 )
     {
         CV_Assert( k == MAT );
@@ -1797,7 +1794,7 @@ Mat& _OutputArray::getMatRef(int i) const
 
 UMat& _OutputArray::getUMatRef(int i) const
 {
-    int k = kind();
+    _InputArray::KindFlag k = kind();
     if( i < 0 )
     {
         CV_Assert( k == UMAT );
@@ -1814,34 +1811,34 @@ UMat& _OutputArray::getUMatRef(int i) const
 
 cuda::GpuMat& _OutputArray::getGpuMatRef() const
 {
-    int k = kind();
+    _InputArray::KindFlag k = kind();
     CV_Assert( k == CUDA_GPU_MAT );
     return *(cuda::GpuMat*)obj;
 }
 std::vector<cuda::GpuMat>& _OutputArray::getGpuMatVecRef() const
 {
-    int k = kind();
+    _InputArray::KindFlag k = kind();
     CV_Assert(k == STD_VECTOR_CUDA_GPU_MAT);
     return *(std::vector<cuda::GpuMat>*)obj;
 }
 
 ogl::Buffer& _OutputArray::getOGlBufferRef() const
 {
-    int k = kind();
+    _InputArray::KindFlag k = kind();
     CV_Assert( k == OPENGL_BUFFER );
     return *(ogl::Buffer*)obj;
 }
 
 cuda::HostMem& _OutputArray::getHostMemRef() const
 {
-    int k = kind();
+    _InputArray::KindFlag k = kind();
     CV_Assert( k == CUDA_HOST_MEM );
     return *(cuda::HostMem*)obj;
 }
 
 void _OutputArray::setTo(const _InputArray& arr, const _InputArray & mask) const
 {
-    int k = kind();
+    _InputArray::KindFlag k = kind();
 
     if( k == NONE )
         ;
@@ -1869,7 +1866,7 @@ void _OutputArray::setTo(const _InputArray& arr, const _InputArray & mask) const
 
 void _OutputArray::assign(const UMat& u) const
 {
-    int k = kind();
+    _InputArray::KindFlag k = kind();
     if (k == UMAT)
     {
         *(UMat*)obj = u;
@@ -1891,7 +1888,7 @@ void _OutputArray::assign(const UMat& u) const
 
 void _OutputArray::assign(const Mat& m) const
 {
-    int k = kind();
+    _InputArray::KindFlag k = kind();
     if (k == UMAT)
     {
         m.copyTo(*(UMat*)obj); // TODO check m.getUMat()
@@ -1922,12 +1919,7 @@ void _OutputArray::move(UMat& u) const
     int k = kind();
     if (k == UMAT)
     {
-#ifdef CV_CXX11
         *(UMat*)obj = std::move(u);
-#else
-        *(UMat*)obj = u;
-        u.release();
-#endif
     }
     else if (k == MAT)
     {
@@ -1962,12 +1954,7 @@ void _OutputArray::move(Mat& m) const
     }
     else if (k == MAT)
     {
-#ifdef CV_CXX11
         *(Mat*)obj = std::move(m);
-#else
-        *(Mat*)obj = m;
-        m.release();
-#endif
     }
     else if (k == MATX)
     {
@@ -1983,7 +1970,7 @@ void _OutputArray::move(Mat& m) const
 
 void _OutputArray::assign(const std::vector<UMat>& v) const
 {
-    int k = kind();
+    _InputArray::KindFlag k = kind();
     if (k == STD_VECTOR_UMAT)
     {
         std::vector<UMat>& this_v = *(std::vector<UMat>*)obj;
@@ -2021,7 +2008,7 @@ void _OutputArray::assign(const std::vector<UMat>& v) const
 
 void _OutputArray::assign(const std::vector<Mat>& v) const
 {
-    int k = kind();
+    _InputArray::KindFlag k = kind();
     if (k == STD_VECTOR_UMAT)
     {
         std::vector<UMat>& this_v = *(std::vector<UMat>*)obj;
